#!/usr/local/bin/wish

# archive.tcl - The Tape Interface
#
# Copyright (c) 1997, 1998 Mark Black
#
#
# This file contains:
#   getTapeHost - Returns a list of tape servers
#   newTape     - Add a new backup tape to the tape server
#   tapeText    - Fill the text window with data
#   fileSelect  - File selection GUI for the backup
#   getDir      - Read the directory on a remote machine
#   resSelect   - File restoration GUI
#   getCat      - Read the tape catalog for a specific volume
#   canBACKUP   - Draw the canvas for the backup tools
#   canRESTORE  - Draw the canvas for the restore tools
#   canTSCHED   - Draw the canvas for the tape scheduling tools


#
# getTapeHost - Read the MAT hosts file and return a list
#               of all tape servers.
#
proc getTapeHost {} {
    global active_hosts
    set retval {}
    foreach l $active_hosts {
	set tmp [ lindex [split $l ":" ] 3 ]
	foreach e [ split $tmp "," ] {
	    if { $e == "TAPE" } {
		# It's a tape host
		lappend retval [ lindex [split $l ":" ] 0 ]
	    }
	}
    }
    return $retval
}


#
# newTape - Get all the information from a user to create a new tape
#           NOTE:  This function does not have to deal with Delete, or MOD
#
proc newTape { tapeserv } {
    global errno
    set host $tapeserv
    if { [winfo exists .ntps ] != 0 } {
	destroy .ntps
    }

    toplevel .ntps -class Dialog
    wm title .ntps "New Tape for $tapeserv"
    wm iconname .ntps "New Tape"
    entry .ntps.hidden1 
    .ntps.hidden1 insert end $tapeserv
    
    frame .ntps.f2 -bd 2 -relief "raised"
    label .ntps.f2.l1 -text "New Tape" 
    pack .ntps.f2.l1 -fill x
    frame .ntps.f2.f1 -bd 2 -relief "flat"
    label .ntps.f2.f1.l1 -text "Label:"
    entry .ntps.f2.f1.e1 -width 20
    pack .ntps.f2.f1.l1 .ntps.f2.f1.e1 -fill x -side left -expand 1
    
    frame .ntps.f2.f2 -bd 2 -relief "flat"
    label .ntps.f2.f2.l1 -text "Size (Meg):" -width 20
    menubutton .ntps.f2.f2.b1 -text "..." -menu .ntps.f2.f2.b1.m -relief raised
    menu .ntps.f2.f2.b1.m -tearoff 0 
    set vals { 120 150 240 1300 2000 2300 2600 4000 7000 20000 35000 40000 }
    foreach val $vals {
	.ntps.f2.f2.b1.m add command -label $val -command ".ntps.f2.f2.e1 delete 0 end ; .ntps.f2.f2.e1 insert end \"$val\" "
    }
    entry .ntps.f2.f2.e1 -width 7
    pack .ntps.f2.f2.l1 .ntps.f2.f2.b1 .ntps.f2.f2.e1 -fill both -side left -expand 1
    
    frame .ntps.f2.f3 -bd 2 -relief "flat"
    label .ntps.f2.f3.l1 -text "Retention (weeks):" -width 20
    menubutton .ntps.f2.f3.b1 -text "..." -menu .ntps.f2.f3.b1.m -relief raised
    menu .ntps.f2.f3.b1.m -tearoff 0 
    set vals { 1 2 4 8 12 16 24 48 }
    foreach val $vals {
	.ntps.f2.f3.b1.m add command -label $val -command ".ntps.f2.f3.e1 delete 0 end ; .ntps.f2.f3.e1 insert end \"$val\" "
    }
    entry .ntps.f2.f3.e1 -width 7
    pack .ntps.f2.f3.l1 .ntps.f2.f3.b1 .ntps.f2.f3.e1 -fill x -side left -expand 1
    
    frame .ntps.f2.f4 -bd 2 -relief "flat"
    label .ntps.f2.f4.l1 -text "Tape Device:" -width 12
    menubutton .ntps.f2.f4.b1 -text "..." -menu .ntps.f2.f4.b1.m -relief raised
    menu .ntps.f2.f4.b1.m -tearoff 0 
    
    set vals {}
    set data [queryHost $tapeserv "MON get tapedevs" 0 ]
    for { set p 0 } { $p < [llength $data] } { incr p 1 } {
	set tmp [lindex $data $p ]
	lappend vals [lindex [split [lindex $data $p ] {:} ] 0 ]
    }
    foreach val $vals {
	.ntps.f2.f4.b1.m add command -label $val -command ".ntps.f2.f4.e1 delete 0 end ; .ntps.f2.f4.e1 insert end \"$val\" "
    }
    entry .ntps.f2.f4.e1 -width 16
    pack .ntps.f2.f4.l1 .ntps.f2.f4.b1 .ntps.f2.f4.e1 -fill x -side left -expand 1

    pack .ntps.f2.f1 .ntps.f2.f2 .ntps.f2.f3 .ntps.f2.f4 -side top -fill both -expand 1
    
    # -----------------------------------------------------------------------------------
    frame .ntps.f7 -bd 2 -relief "raised"
    button .ntps.f7.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	set cmd "TAPE newvol [.ntps.hidden1 get] 0 [.ntps.f2.f2.e1 get] [.ntps.f2.f4.e1 get] [.ntps.f2.f1.e1 get]"
	
	set returnval [queryHost [.ntps.hidden1 get] $cmd 1 ]
	if { $errno != 0 } {
	    error 2001
	    puts stdout "ERROR:  Unable to add new tape to $tapeserv"
	} else {
	    repack 121
	    destroy .ntps
	}
    }
    button .ntps.f7.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	destroy .ntps
    }
    button .ntps.f7.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	help newtape
    }
    pack .ntps.f7.b1 .ntps.f7.b2 .ntps.f7.b3 -side left -fill x 
    
    pack .ntps.f2 .ntps.f7 -side top -fill both -expand 1
}


#
# -- Tape Specific Procedure to read data from raw data and fill list window --
#    w = window to display it in
proc tapeText { data w type } {
    global currenthost
    global tapeserv
    . config -cursor watch
    $w config -cursor watch
    $w configure -state normal
    $w delete 0.0 end

    # Backup
    set tabbing(121) {6 16 24 }
    set toplabel(121) "ID    Name            Contents"
    # Restore 
    set tabbing(131) {6 16 24 }
    set toplabel(131) "ID    Name            Contents"
    # Tape Devices
    set tabbing(161) {32 24 10 }
    set toplabel(161) "Tape Device Name                Device                  # of Slots"

    set bold "-background #c0c000 -relief raised -borderwidth 1"
    set normal "-background {} -relief flat"
    set select "-underline 1"
    set tplabel $w
    append tplabel "label"
    $tplabel configure -state normal
    $tplabel delete 0.0 end
    $tplabel insert end $toplabel($type)
    $tplabel configure -state disabled

    if { $type == 121 } {
	# ------------------------------  Backup  --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {;} ]
#	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 1 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 6 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lrange $tmp3 11 end ]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1>  "fileSelect $currenthost [lindex $tmp3 1 ] 0" 
	}
	$w configure -state disabled
    } elseif { $type == 131 } {
	# ------------------------------  Restore  --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {;} ]
	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 1 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 6 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lrange $tmp3 11 end ]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1> \
		    "resSelect $currenthost [lindex $tmp3 1 ] \"[lindex $tmp3 6 ]\" \"[string trim [lrange $tmp3 11 end ]]\"" 
	}
	$w configure -state disabled
    } elseif { $type == 161 } {
	# ------------------------------  Tape Devices  --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {:} ]
	    set tmp4 [lindex $tmp3 0]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 1]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 2]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1> \
		    "resSelect $currenthost [lindex $tmp3 1 ] \"[lindex $tmp3 6 ]\" \"[string trim [lrange $tmp3 11 end ]]\"" 
	}
	$w configure -state disabled
    }
}


# Display a series of listboxes and allow users to select the
# desired directories and files.
set sd {}
proc fileSelect { chost tapeid flag } {
    global sd
    global errno
    global tapeserv
    set sd "/"
    if { [winfo exists .fsel ] == 0 } {
	toplevel .fsel -class Dialog
	wm title .fsel "Backup $chost to $tapeserv"
	wm iconname .fsel "Backup"
	
	entry .fsel.hidden1
	.fsel.hidden1 insert end $tapeid
	entry .fsel.hidden2
	.fsel.hidden2 insert end $chost
	entry .fsel.h3
	.fsel.h3 insert end $flag

	frame .fsel.f2 -bd 2 -relief "raised"
	label .fsel.f2.l1 -text "Select Files or Directories to Backup"
	pack .fsel.f2.l1 -fill x

	frame .fsel.f6 -bd 2 -relief "raised"
	
	frame .fsel.f6.f2 -bd 2 -relief "flat"
	scrollbar .fsel.f6.f2.vscroll -orient vertical -command ".fsel.f6.f2.text yview"
	scrollbar .fsel.f6.f2.hscroll -orient horizontal -command ".fsel.f6.f2.text xview"
	listbox .fsel.f6.f2.text -selectforeground white -selectbackground #000080 \
		-yscrollcommand {.fsel.f6.f2.vscroll set } \
		-xscrollcommand {.fsel.f6.f2.hscroll set } -height 10 -width 32
	.fsel.f6.f2.text config -cursor hand2
	pack .fsel.f6.f2.vscroll -side left -fill y
	pack .fsel.f6.f2.hscroll -side bottom -fill x
	pack .fsel.f6.f2.text -side left -expand 1 -fill both

	frame .fsel.f6.f4 -bd 2 -relief "flat"
	button .fsel.f6.f4.b1 -text "Add" -command {
	    if { [.fsel.f6.f2.text curselection ] != "" } {
		.fsel.f6.f6.text insert end [.fsel.f6.f2.text get [.fsel.f6.f2.text curselection ]]
	    }
	}
	button .fsel.f6.f4.b2 -text "Remove" -command {
	    if { [.fsel.f6.f6.text curselection ] != "" } {
		.fsel.f6.f6.text delete [.fsel.f6.f6.text curselection ]
	    }
	}
	pack .fsel.f6.f4.b1 .fsel.f6.f4.b2 -side top -fill both -anchor center 
	
	frame .fsel.f6.f6 -bd 2 -relief "flat"
	scrollbar .fsel.f6.f6.vscroll -orient vertical -command ".fsel.f6.f6.text yview" 
	scrollbar .fsel.f6.f6.hscroll -orient horizontal -command ".fsel.f6.f6.text xview"
	listbox .fsel.f6.f6.text -selectforeground white -selectbackground #000080 \
		-yscrollcommand {.fsel.f6.f6.vscroll set } \
		-xscrollcommand {.fsel.f6.f6.hscroll set } -height 10 -width 32
	.fsel.f6.f6.text config -cursor hand2
	pack .fsel.f6.f6.vscroll -side left -fill y
	pack .fsel.f6.f6.hscroll -side bottom -fill x
	pack .fsel.f6.f6.text -side left -expand 1 -fill both

	pack .fsel.f6.f2 .fsel.f6.f4 .fsel.f6.f6 -side left -fill both
	
	frame .fsel.f8 -bd 2 -relief "flat"
	button .fsel.f8.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	    if {[.fsel.h3 get] == 1} {
		# Called from the backup scheduler
		.matdjt.f3.f4.text delete 0 end
		foreach i [ .fsel.f6.f6.text get 0 end] {
		    .matdjt.f3.f4.text insert end $i
		}
		destroy .fsel
	    } else {
		set act [ actionItem .fsel "Insert the backup tape in the drive" {Ready Cancel} blue ]
		if { $act == 0 } {
		    puts stdout "Starting Backup"
		    set tmp "TAPE backup $tapeserv [.fsel.hidden1 get] 0 "
		    foreach fsys [.fsel.f6.f6.text get 0 end] {
			append tmp "[string trimright [string trimright $fsys] {/}],"
		    }
		    set cmd [string trimright $tmp "," ] 
		    puts stdout "Command = $cmd"
		    #		set returnval [queryHost [.fsel.hidden2 get] $cmd 1 ]
		    if { $errno != 0 } {
			error 2002
			puts stdout "ERROR:  Unable to backup of $fsys"
		    } else {
			repack 121
			destroy .fsel
		    }
		}
	    }
	}
	button .fsel.f8.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	    destroy .fsel
	}
	button .fsel.f8.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	    help backup
	}
	pack .fsel.f8.b1 .fsel.f8.b2 .fsel.f8.b3 -anchor w -side left -fill x 

	pack .fsel.f2 .fsel.f6 .fsel.f8 -fill both -side top

	# ------------ Bindings --------------------
	bind .fsel.f6.f2.text <Button-3> {
	    .fsel.f6.f6.text insert end [.fsel.f6.f2.text get @%x,%y ]
	}
	bind .fsel.f6.f6.text <Button-3> {
	    .fsel.f6.f6.text delete @%x,%y
	}
	bind .fsel.f6.f6.text <Double-1> {
	    if { [.fsel.f6.f6.text curselection ] != "" } {
		.fsel.f6.f6.text delete [.fsel.f6.f6.text curselection ]
	    }
	}
	bind .fsel.f6.f2.text <Double-1> {
	    if { [.fsel.f6.f2.text curselection ] != {} } {
		set t [.fsel.f6.f2.text get [.fsel.f6.f2.text curselection ] ]
		if { $t == ".." } {
		    # Need to go down a directory
		    global sd
		    set ldir [string last {/} $sd ]
		    if { $ldir <= 0 } {
			set sd "/"
		    } else {
			puts stdout "ldir = $ldir"
			set tmp [string range $sd 0 $ldir ]
			set sd [string trimright $tmp {/} ]
		    }
		    getDir $sd .fsel.f6.f2.text
		} elseif { [string range $t end end ] == {/} } {
		    global sd
		    set sd [string trimright [.fsel.f6.f2.text get [.fsel.f6.f2.text curselection ] ] {/} ]
		    getDir $sd .fsel.f6.f2.text
		}
	    }
	}

    }

    .fsel.f6.f2.text delete 0 end
    .fsel.f6.f6.text delete 0 end

    # Get directory information
    getDir $sd .fsel.f6.f2.text

    if { $flag == 1} {
	# Read the file list from the backup scheduler window
	foreach i [ .matdjt.f3.f4.text get 0 end ] {
	    .fsel.f6.f6.text insert end $i
	}
    }
}


# getDir  - Read a directory from the remote machine and
#           populate the listbox $lb.  If $dir is NOT "/"
#           then include ".." in the listbox.
proc getDir { dir lb } {
    global currenthost
    global errno
    $lb delete 0 end
 
    set cmd "TAPE prtdir $currenthost $dir"
    set retval [queryHost $currenthost $cmd 0 ]
    if { $errno != 0 } {
	error 402
	puts stdout "ERROR:  Unable to retrieve directory information from $currenthost"
    }
    
    if { $dir != "/" } {
	$lb insert end ".."
    }
    foreach e $retval {
	$lb insert end $e
    }
}


set volno 1
# resSelect  - Display a series of listboxes and allow users to select the
#              desired backup set and directories and files.
proc resSelect { chost tapeid tapename data} {
    global errno
    global tapeserv
    global volno
    set volno 1
    set sd "/"
    if { [winfo exists .ress ] == 0 } {
	toplevel .ress -class Dialog
	wm title .ress "Restore to $chost from $tapeserv"
	wm iconname .ress "Restore"
	entry .ress.hidden1
	entry .ress.hidden2
	entry .ress.h3
	# Hidden entry for the current path
	entry .ress.h4
	.ress.hidden1 insert end $tapeid
	.ress.hidden2 insert end $tapename
	.ress.h3 insert end $chost
	.ress.h4 insert end "/"

	frame .ress.f2 -bd 2 -relief "raised"
	label .ress.f2.l1 -text "Backup volumes available on $tapename"
	pack .ress.f2.l1 -fill x

	frame .ress.f4 -bd 2 -relief "raised"
	frame .ress.f4.f2 -bd 2 -relief "flat"
	label .ress.f4.f2.l1 -text "Restore from volume"
	pack .ress.f4.f2.l1 -side top -fill x
	scrollbar .ress.f4.f2.vscroll -orient vertical -command ".ress.f4.f2.text yview"
	scrollbar .ress.f4.f2.hscroll -orient horizontal -command ".ress.f4.f2.text xview"
	listbox .ress.f4.f2.text -selectforeground white -selectbackground #000080 \
		-yscrollcommand {.ress.f4.f2.vscroll set } \
		-xscrollcommand {.ress.f4.f2.hscroll set } -height 3 -width 32
	pack .ress.f4.f2.vscroll -side left -fill y
	pack .ress.f4.f2.hscroll -side bottom -fill x
	pack .ress.f4.f2.text -side left -expand 1 -fill both

	frame .ress.f4.f4 -bd 2 -relief "flat"
	frame .ress.f4.f4.f1 -bd 2 -relief "flat"
	label .ress.f4.f4.f1.l1 -text "Creation Date" -width 14
	entry .ress.f4.f4.f1.e1 -width 32
	pack .ress.f4.f4.f1.l1 .ress.f4.f4.f1.e1 -side left -fill x
	frame .ress.f4.f4.f2 -bd 2 -relief "flat"
	label .ress.f4.f4.f2.l1 -text "Contents" -width 14
	entry .ress.f4.f4.f2.e1 -width 32
	pack .ress.f4.f4.f2.l1 .ress.f4.f4.f2.e1 -side left -fill x
	pack .ress.f4.f4.f1 .ress.f4.f4.f2 -side top -fill both
	.ress.f4.f4.f1.e1 configure -state disabled
	.ress.f4.f4.f2.e1 configure -state disabled

	pack .ress.f4.f2 .ress.f4.f4 -side left -fill x 
	frame .ress.f6 -bd 2 -relief "raised"
	
	frame .ress.f6.f2 -bd 2 -relief "flat"
	scrollbar .ress.f6.f2.vscroll -orient vertical -command ".ress.f6.f2.text yview"
	scrollbar .ress.f6.f2.hscroll -orient horizontal -command ".ress.f6.f2.text xview"
	listbox .ress.f6.f2.text -selectforeground white -selectbackground #000080 \
		-yscrollcommand {.ress.f6.f2.vscroll set } \
		-xscrollcommand {.ress.f6.f2.hscroll set } -height 10 -width 32
	.ress.f6.f2.text config -cursor hand2
	pack .ress.f6.f2.vscroll -side left -fill y
	pack .ress.f6.f2.hscroll -side bottom -fill x
	pack .ress.f6.f2.text -side left -expand 1 -fill both

	frame .ress.f6.f4 -bd 2 -relief "flat"
	button .ress.f6.f4.b1 -text "Add" -command {
	    if { [.ress.f6.f2.text curselection ] != "" } {
		.ress.f6.f6.text insert end [.ress.f6.f2.text get [.ress.f6.f2.text curselection ]]
	    }
	}
	button .ress.f6.f4.b2 -text "Remove" -command {
	    if { [.ress.f6.f6.text curselection ] != "" } {
		.ress.f6.f6.text delete [.ress.f6.f6.text curselection ]
	    }
	}
	pack .ress.f6.f4.b1 .ress.f6.f4.b2 -side top -fill both -anchor center 
	
	frame .ress.f6.f6 -bd 2 -relief "flat"
	scrollbar .ress.f6.f6.vscroll -orient vertical -command ".ress.f6.f6.text yview" 
	scrollbar .ress.f6.f6.hscroll -orient horizontal -command ".ress.f6.f6.text xview"
	listbox .ress.f6.f6.text -selectforeground white -selectbackground #000080 \
		-yscrollcommand {.ress.f6.f6.vscroll set } \
		-xscrollcommand {.ress.f6.f6.hscroll set } -height 10 -width 32
	.ress.f6.f6.text config -cursor hand2
	pack .ress.f6.f6.vscroll -side left -fill y
	pack .ress.f6.f6.hscroll -side bottom -fill x
	pack .ress.f6.f6.text -side left -expand 1 -fill both

	
	pack .ress.f6.f2 .ress.f6.f4 .ress.f6.f6 -side left -fill both
	
	frame .ress.f8 -bd 2 -relief "flat"
	button .ress.f8.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	    set act [ actionItem .ress "Insert the backup tape \"[.ress.hidden2 get]\" in the drive." {Ready Cancel} blue ]
	    if { $act == 0 } {
		puts stdout "Starting Restore"
		global volno
		# Tapevol = 1 for first
		set tmp "TAPE restore $tapeserv [.ress.hidden1 get] $volno 0 "
		foreach fsys [.ress.f6.f6.text get 0 end] {
		    append tmp "[string trimright [string trimright $fsys] {/}],"
		}
		set cmd [string trimright $tmp "," ] 
		puts stdout "Command = $cmd"
#		set returnval [queryHost [.ress.h3 get] $cmd 1 ]
		if { $errno != 0 } {
		    error 2003
		    puts stdout "ERROR:  Unable to restore $fsys"
		} else {
		    repack 131
		    destroy .ress
		}

	    }
	}
	button .ress.f8.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	    destroy .ress
	}
	button .ress.f8.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	    help backup
	}
	pack .ress.f8.b1 .ress.f8.b2 .ress.f8.b3 -anchor w -side left -fill x 

	pack .ress.f2 .ress.f4 .ress.f6 .ress.f8 -fill both -side top

	# ------------ Bindings --------------------
	bind .ress.f6.f2.text <Button-3> {
	    .ress.f6.f6.text insert end [.ress.f6.f2.text get @%x,%y ]
	}
	bind .ress.f6.f6.text <Button-3> {
	    .ress.f6.f6.text delete @%x,%y
	}
	bind .ress.f6.f6.text <Double-1> {
	    if { [.ress.f6.f6.text curselection ] != "" } {
		.ress.f6.f6.text delete [.ress.f6.f6.text curselection ]
	    }
	}
	bind .ress.f6.f2.text <Double-1> {
	    if { [.ress.f6.f2.text curselection ] != {} } {
		set t [.ress.f6.f2.text get [.ress.f6.f2.text curselection ] ]
		if { $t == ".." } {
		    # Need to go down a directory
		    global sd
		    set ldir [string last {/} $sd ]
		    if { $ldir <= 0 } {
			set sd "/"
		    } else {
			puts stdout "ldir = $ldir"
			set tmp [string range $sd 0 $ldir ]
			set sd [string trimright $tmp {/} ]
		    }
		    global volno
		    getCat [.ress.hidden1 get] $volno $sd .ress.f6.f2.text
		} elseif { [string range $t end end ] == {/} } {
		    global sd
		    set sd [string trimright [.ress.f6.f2.text get [.ress.f6.f2.text curselection ] ] {/} ]
		    getCat [.ress.hidden1 get] $volno $sd .ress.f6.f2.text
		}
	    }
	}

    }

    .ress.f6.f2.text delete 0 end
    .ress.f6.f6.text delete 0 end
    .ress.f4.f2.text delete 0 end
    .ress.f4.f4.f1.e1 delete 0 end
    .ress.f4.f4.f2.e1 delete 0 end

    # Populate tape volumes list
    foreach vl $data {
	.ress.f4.f2.text insert end $vl
    }


    # Bindings
    bind .ress.f4.f2.text <Double-1> {
	if { [.ress.f4.f2.text curselection ] != "" } {
	    global volno
	    .ress.f4.f4.f1.e1 configure -state normal
	    .ress.f4.f4.f2.e1 configure -state normal
	    .ress.f4.f4.f1.e1 delete 0 end
	    .ress.f4.f4.f2.e1 delete 0 end
	    .ress.f4.f4.f2.e1 insert end [.ress.f4.f2.text get [.ress.f4.f2.text curselection ]]
	    set volno [expr [.ress.f4.f2.text curselection ] + 1 ]
	    .ress.f4.f4.f1.e1 configure -state disabled
	    .ress.f4.f4.f2.e1 configure -state disabled
	    .ress.h4 delete 0 end
	    .ress.h4 insert end [.ress.f4.f4.f2.e1 get ]
	    getCat [.ress.hidden1 get] $volno [.ress.h4 get] .ress.f6.f2.text
	    .ress.f6.f6.text delete 0 end
	}

    }

}




# getCat  - Read a catalog from the remote machine and
#           populate the listbox $lb.  If $dir is NOT "/"
#           then include ".." in the listbox.
proc getCat { tapeid volno dir lb } {
    global currenthost
    global tapeserv
    global errno
    $lb delete 0 end
    set cmd "TAPE prtcat $tapeserv $tapeid $volno $dir"
    puts stdout "Cmd = $cmd"
    set retval [queryHost $currenthost $cmd 0 ]
    if { $errno != 0 } {
	error 402
	puts stdout "ERROR:  Unable to retrieve directory information from $currenthost"
    }
    
    if { $dir != "/" } {
	$lb insert end ".."
    }
    foreach e $retval {
	$lb insert end $e
    }
}


#
# canBACKUP  - Fill canvas with tape backup icons
#
proc canBACKUP {} {
    # ---------------------  Tape Backup Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Backup "
    set level 120
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(121) [ image create photo -file "../lib/new.gif" ]
    set fig(122) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0
    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    set node [ .tp.host.canvas create image $startX $startY -image $fig(121) -tags "New" ]
    .tp.host.canvas bind "New" <Double-1> "set browse 90 ;repack 90"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Add Server" -tags "New" \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    
    # Draw icons for each Tape server
    foreach tapeS [ getTapeHost ] {
	set node [ .tp.host.canvas create image $startX $startY -image $fig(122) -tags $tapeS ]
	.tp.host.canvas bind $tapeS <Double-1> "set tapeserv $tapeS ;set browse 121 ;repack 121 "
	set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
		-justify center -text "$tapeS" -tags nameLabel \
		-font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
	incr startX $incX 
	if { $cntx == 8 } {
	    set startX 40
	    incr startY $incY 
	    incr cntx 1
	}
    }
}


#
# canRESTORE  - Fill canvas with tape restore icons
#
proc canRESTORE {} {
    # ---------------------  Tape Restore Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Restore "
    set level 130
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(132) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0
    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    
    # Draw icons for each Tape server
    foreach tapeS [ getTapeHost ] {
	set node [ .tp.host.canvas create image $startX $startY -image $fig(132) -tags $tapeS ]
	.tp.host.canvas bind $tapeS <Double-1> "set tapeserv $tapeS ;repack 131 "
	set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
		    -justify center -text "$tapeS" -tags nameLabel \
		    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
	incr startX $incX 
	if { $cntx == 8 } {
	    set startX 40
	    incr startY $incY 
	    incr cntx 1
	}
    }
}


#
# canTSCHED  - Tape scheduling window
#
proc canTSCHED {} {
# ---------------------  Tape Restore Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Restore "
    set level 130
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(132) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0
    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    
    # Draw icons for each Tape server
    foreach tapeS [ getTapeHost ] {
	set node [ .tp.host.canvas create image $startX $startY -image $fig(132) -tags $tapeS ]
	.tp.host.canvas bind $tapeS <Double-1> "set tapeserv $tapeS ;repack 131 "
	set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
		    -justify center -text "$tapeS" -tags nameLabel \
		    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
	incr startX $incX 
	if { $cntx == 8 } {
	    set startX 40
	    incr startY $incY 
	    incr cntx 1
	}
    }
}